/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: tpinit.c
 *
 * Purpose: This is the target process black box init program.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Id: tpinit.c,v 1.3 1994/02/11 13:37:08 irsuprt Exp $
 * $Log: tpinit.c,v $
 * Revision 1.3  1994/02/11  13:37:08  irsuprt
 * indef len
 *
 * Revision 1.3  1994/02/11  13:37:08  irsuprt
 * indef len
 *
 * Revision 1.2  1993/11/02  15:43:50  irsuprt
 * changed #ifdef 0 to #ifdef ACTIVE_TEMPORARY of line 446 for ANSI compliace
 * (PROBLEM 13)
 *
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 *
 *****************************************************************************/

#include <stdio.h>
#include <rpc/rpc.h>
#include <fcntl.h>
#include <pwd.h>
#include <ctype.h>
#include <sys/errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include "tp.h"
#include "asn_incl.h"
#include "ir_oid.h"
#include "irerror.h"
#include "tpconst.h"
#include "tpbbox.h"
#include "common.h"

/**************************** Global Ids *********************************/

/**************************** Extern Global Ids **************************/

extern int errno;

/*****************************************************************************
 *
 * Function: tp_fill_init_response
 *
 * Purpose: This function fills and creates the init response PDU.
 *
 * Input Arguments:
 *              PDU_ireq - init request PDU
 *              accept - 1 (init response filled with Accept)
 *                       0 (init response filled with reject)
 *              protocol_version - replied protocol version
 *              options - replied options
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              PDU_iresp - init response PDU pointer
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_fill_init_response(PDU_ireq, accept, protocol_version, options, PDU_iresp)
	APDU *PDU_ireq;
    int accept;
    char protocol_version;
    char options;
    APDU *PDU_iresp;
{

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX: calling tp_fill_init_response\n");
#endif

    PDU_iresp->a.initResp = (struct InitializeResp *) Asn1Alloc(sizeof(struct InitializeResp));

    PDU_iresp->choiceId = APDU_INITRESP;

	/* filling referenceId field */
    PDU_iresp->a.initResp->referenceId.octetLen = PDU_ireq->a.initReq->referenceId.octetLen;
    PDU_iresp->a.initResp->referenceId.octs = (char *) Asn1Alloc(PDU_ireq->a.initReq->referenceId.octetLen);
	if (PDU_iresp->a.initResp->referenceId.octetLen > 0) 
		memcpy(PDU_iresp->a.initResp->referenceId.octs, PDU_ireq->a.initReq->referenceId.octs, PDU_ireq->a.initReq->referenceId.octetLen);
	else
		PDU_iresp->a.initResp->referenceId.octs = NULL;

#ifdef DEBUG_LEVEL3
    fprintf(DEBUG_FILE3, "BBOX: tp_fill_init_response: referenceId.octetLen=%d\n", PDU_iresp->a.initResp->referenceId.octetLen);
    fprintf(DEBUG_FILE3, "BBOX: tp_fill_init_response: referenceId.octs=%s/\n", PDU_iresp->a.initResp->referenceId.octs);
#endif

    PDU_iresp->a.initResp->protocolVersion.bitLen = 1;
    PDU_iresp->a.initResp->protocolVersion.bits = Asn1Alloc(1);
    PDU_iresp->a.initResp->protocolVersion.bits[0] = protocol_version;

    PDU_iresp->a.initResp->options.bitLen = 1;
    PDU_iresp->a.initResp->options.bits = Asn1Alloc(1);
    /* support 0-search, 1-present, 2-delete, 3-resource report, */
    /* 4-trig resc ctrl, 5-resc ctrl, 6-access control. */
    PDU_iresp->a.initResp->options.bits[0] = options;

    PDU_iresp->a.initResp->preferredMessageSize = MAXMESSAGESIZE;
    PDU_iresp->a.initResp->maximumRecordSize = MAXMESSAGESIZE;
    if (accept)
        PDU_iresp->a.initResp->result = TRUE;        /* accept */
    else
        PDU_iresp->a.initResp->result = FALSE;        /* reject */

    PDU_iresp->a.initResp->implementationId.octetLen = sizeof(IMPLEMENTATION_ID) - 1;
    PDU_iresp->a.initResp->implementationId.octs = (char *) Asn1Alloc(PDU_iresp->a.initResp->implementationId.octetLen);
    memcpy(PDU_iresp->a.initResp->implementationId.octs, IMPLEMENTATION_ID, PDU_iresp->a.initResp->implementationId.octetLen);

    PDU_iresp->a.initResp->implementationName.octetLen = sizeof(IMPLEMENTATION_NAME) - 1;
    PDU_iresp->a.initResp->implementationName.octs = (char *) Asn1Alloc(PDU_iresp->a.initResp->implementationName.octetLen);
    memcpy(PDU_iresp->a.initResp->implementationName.octs, IMPLEMENTATION_NAME, PDU_iresp->a.initResp->implementationName.octetLen);

    PDU_iresp->a.initResp->implementationVersion.octetLen = sizeof(IMPLEMENTATION_VERSION) - 1;
    PDU_iresp->a.initResp->implementationVersion.octs = (char *) Asn1Alloc(PDU_iresp->a.initResp->implementationVersion.octetLen);
    memcpy(PDU_iresp->a.initResp->implementationVersion.octs, IMPLEMENTATION_VERSION, PDU_iresp->a.initResp->implementationVersion.octetLen);

	/* filling userInfoField field */
    PDU_iresp->a.initResp->userInfoField = Asn1Alloc(sizeof(UserInfoField));
	ASN_MakeOid((char *) OID_CLASS_USER_INFO_FIELD, &PDU_iresp->a.initResp->userInfoField->direct_reference);
    PDU_iresp->a.initResp->userInfoField->indirect_reference = Asn1Alloc(sizeof(AsnInt));
    PDU_iresp->a.initResp->userInfoField->indirect_reference = 0;
    PDU_iresp->a.initResp->userInfoField->data_value_descriptor.octs = Asn1Alloc(sizeof("Dummy UserInfo"));
    PDU_iresp->a.initResp->userInfoField->data_value_descriptor.octetLen = sizeof("Dummy UserInfo");
    PDU_iresp->a.initResp->userInfoField->encoding = Asn1Alloc(sizeof(struct EXTERNALChoice));
    PDU_iresp->a.initResp->userInfoField->encoding->choiceId = EXTERNALCHOICE_OCTET_ALIGNED;
    PDU_iresp->a.initResp->userInfoField->encoding->a.octet_aligned = Asn1Alloc(sizeof(AsnOcts));
    PDU_iresp->a.initResp->userInfoField->encoding->a.octet_aligned->octs = Asn1Alloc(sizeof("Dummy UserInfo"));
    PDU_iresp->a.initResp->userInfoField->encoding->a.octet_aligned->octetLen = sizeof("Dummy UserInfo");

    return (SUCCESS);

}

/*****************************************************************************
 *
 * Function: tp_create_init_response
 *
 * Purpose: This function creates the init response PDU, encodes, and
 *          creates the init response APDU for sending to IR kernel.
 *
 * Input Arguments:
 *              PDU_ireq - init request PDU
 *              accept - 1 (init response filled with Accept)
 *                       0 (init response filled with reject)
 *              protocol_version - replied protocol version
 *              options - replied options
 *              data_ptr - temporary data buffer for encoding process
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              outputBuf - encoded APDU pointer
 *              APDU_length - the size of the encoded APDU
 *
 * External Variables:
 *
 * Function used:
 *              tp_fill_init_response()
 *              SBufInstallData()
 *              SBufResetInWriteRvsMode()
 *              BEncAPDU()
 *              ir_fwrite()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_create_init_response(PDU_ireq, accept, protocol_version, options, data_ptr, outputBuf, APDU_length)
	APDU *PDU_ireq;
    int accept;                        /* boolean */
    char protocol_version;
    char options;
    char **data_ptr;
    SBuf *outputBuf;
    int *APDU_length;
{
    APDU PDU_ptr;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX:Calling tp_create_init_response()\n");
#endif

    tp_fill_init_response(PDU_ireq, accept, protocol_version, options, &PDU_ptr);

    SBufInstallData(outputBuf, (*data_ptr), MAXMESSAGESIZE * 2);
    SBufResetInWriteRvsMode(outputBuf);

    *APDU_length = BEncAPDU(outputBuf, &PDU_ptr);

	if (SBufWriteError(outputBuf))
	{
		display_error("KERNEL", ERR_ENCODE_APDU, -1);
		return(FAILED);
	}

#ifdef DEBUG_LEVEL2
    fprintf(DEBUG_FILE2, "BBOX:Init Response APDU Dump from tpinit\n");
    ir_fwrite(SBufDataPtr(outputBuf), SBufDataLen(outputBuf), 1, DEBUG_FILE2);
#endif

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: tp_check_init_request
 *
 * Purpose: This function checks the init request PDU to make sure the
 *          mandatory fields are present and valid.
 *
 * Input Arguments: PDU_ptr - init request PDU pointer
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              protocol_version - final protocol version
 *              options - final options
 *              check_password_already - indicating if the password has been
 *                                       verified.
 *
 * External Variables:
 *
 * Function used:
 *              display_error()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_check_init_request(PDU_ptr, protocol_version, options, check_password_already)
    APDU *PDU_ptr;
    char *protocol_version;
    char *options;
    int *check_password_already;
{

    int i;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX: calling tp_check_init_request\n");
#endif

/* Initialize the Ids */
*check_password_already = FALSE;

#ifdef ENFORCE_ACCESS_RESOURCE_CONTROL

#ifdef DEBUG_LEVEL1
	fprintf(DEBUG_FILE1, "IR TARGET ENFORCES ACCESS/RESOURCE CONTROL\n");
#endif

    *protocol_version = (char) 0xc0;        /* SR+IR */
    *options = (char) 0xfe;                /* all on */
    ACCESS_CONTROL_REQUIRED = TRUE;
    RESOURCE_CONTROL_REQUIRED = TRUE;

	if (tp_valid_user(PDU_ptr, CHECK_INIT_PASSWORD))
	{
#ifdef DEBUG_LEVEL1
		fprintf(DEBUG_FILE1, "BBOX: tp_check_init_request: ACCESS CONTROL PASSWORD IS OKAY\n");
#endif
		*check_password_already = TRUE;
	}
	else
	{
#ifdef DEBUG_LEVEL1
		fprintf(DEBUG_FILE1, "BBOX: tp_check_init_request: ACCESS CONTROL PASSWORD IS BAD\n");
#endif
	}

#else

#ifdef DEBUG_LEVEL1
	fprintf(DEBUG_FILE1, "BBOX: ACCESS/RESOURCE CONTROL IS NOT ENFORCED\n");
#endif

    /*
     * if the client is a SR system and the access and resource control are
     * not enforced, downgrade to SR.
     */

    if ((((PDU_ptr->a.initReq->protocolVersion.bits[0] & SR_PROTOCOL_BIT) == SR_PROTOCOL_BIT) && ((PDU_ptr->a.initReq->protocolVersion.bits[0] & IR_PROTOCOL_BIT) == 0)) ||
            (((PDU_ptr->a.initReq->options.bits[0] & ACCESS_CONTROL_BIT) == 0) || ((PDU_ptr->a.initReq->options.bits[0] & RESOURCE_CONTROL_BIT) == 0))
        )
    {
        /* downgrade to SR */
        *protocol_version = PDU_ptr->a.initReq->protocolVersion.bits[0];
        *options = PDU_ptr->a.initReq->options.bits[0];
        ACCESS_CONTROL_REQUIRED = FALSE;
        RESOURCE_CONTROL_REQUIRED = FALSE;

#ifdef DEBUG_LEVEL1
        fprintf(DEBUG_FILE1, "BBOX: check_init_req: down grade to SR\n");
#endif

    }
    else if ((PDU_ptr->a.initReq->protocolVersion.bits[0] & IR_PROTOCOL_BIT) == IR_PROTOCOL_BIT)
    {
        /* if IR */
        *protocol_version = (char) 0xc0;        /* SR+IR */
        *options = (char) 0xfe;        /* all on */
        ACCESS_CONTROL_REQUIRED = TRUE;
        RESOURCE_CONTROL_REQUIRED = TRUE;

        *check_password_already = FALSE;
		if (tp_valid_user(PDU_ptr, CHECK_INIT_PASSWORD))
			*check_password_already = TRUE;

#ifdef DEBUG_LEVEL1
        fprintf(DEBUG_FILE1, "BBOX: check_init_req: IR protocol\n");
#endif

    }
    else
    {
        display_error("BBOX", ERR_INVALID_INIT_PROTOCOL_VERSION, -1);
        return (FAILED);
    }
#endif /* end of #ifdef ENFORCE_ACCESS_RESOURCE_CONTROL */

#ifdef ACTIVE_TEMPORARY
    if (PDU_ptr->a.initReq->preferredMessageSize > MAXMESSAGESIZE)
    {
        display_error("BBOX", ERR_INVALID_INIT_MESS_SIZE_REQUEST, -1);
        return (FAILED);
    }
#endif

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: tp_process_init_request
 *
 * Purpose: This function performs the following functions:
 *              - checks the init request PDU,
 *              - performs the security checks,
 *              - performs the resource verification, and
 *              - creates and sends the init response APDU to IR kernel.
 *
 * Input Arguments:
 *              PDU - init request PDU pointer
 *              data_ptr - temporary data pointer for ASN functions
 *              socketFd - the socket Fd for communicating to IR kernel
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *              tp_check_init_request()
 *              FreeDeleteResultSetReq()
 *              tp_access_control()
 *              tp_resource_control()
 *              tp_create_init_response()
 *              tpa_send_APDU()
 *              display_error()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_process_init_request(PDU_ireq, data_ptr, socketFd)
    APDU *PDU_ireq;
    char **data_ptr;
    int socketFd;
{
    int result;
    long APDU_length;
    SBuf APDU_ptr;
    char protocol_version, options;
    int check_password_already;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX: calling tp_process_init_request\n");
#endif

    result = tp_check_init_request(PDU_ireq, &protocol_version, &options, &check_password_already);

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX: tp_process_init_request: check_password_already=%d\n", check_password_already);
#endif

    if (result == SUCCESS)
    {
        if (!check_password_already)
            result = tp_access_control(socketFd, data_ptr, PDU_ireq->a.initReq->referenceId.octetLen, PDU_ireq->a.initReq->referenceId.octs);
        else
            result = SUCCESS;        /* password has been verified */

        if (result != SUCCESS)
        {
            display_error("BBOX", ERR_ACCESS_CONTROL_FAILED, -1);
            tp_create_init_response(PDU_ireq, FALSE, protocol_version, options, data_ptr, &APDU_ptr, &APDU_length);
        }
        else
        {
            if (tp_resource_control(socketFd, data_ptr, PDU_ireq->a.initReq->referenceId.octetLen, PDU_ireq->a.initReq->referenceId.octs) == SUCCESS)
            {
                /* accepted init request */
				tp_create_init_response(PDU_ireq, TRUE, protocol_version, options, data_ptr, &APDU_ptr, &APDU_length);
            }
            else
            {
                display_error("BBOX", ERR_RESOURCE_CONTROL_FAILED, -1);
				tp_create_init_response(PDU_ireq, FALSE, protocol_version, options, data_ptr, &APDU_ptr, &APDU_length);
            }
        }
    }
    else
    {
        /* rejected init request */
		tp_create_init_response(PDU_ireq, FALSE, protocol_version, options, data_ptr, &APDU_ptr, &APDU_length);
    }

    if (tpa_send_APDU(socketFd, &APDU_ptr, APDU_length) <= 0)
    {
        display_error("TP_BBOX", ERR_SEND_DATA_TO_IR_FROM_TP, errno);
        return (FAILED);
    }

    return (SUCCESS);
}
